@@ -65,19 +65,19 @@ _docstring_substitution_definitions = {
     "{{regrid Implementation}}": """**Implementation**
 
         The interpolation is carried out using regridding weights
-        calculated by the `ESMF` package, a Python interface to the
+        calculated by the `esmpy` package, a Python interface to the
         Earth System Modeling Framework (ESMF) regridding utility:
-        `https://earthsystemmodeling.org/regrid`_. Outside of `ESMF`,
+        `https://earthsystemmodeling.org/regrid`_. Outside of `esmpy`,
         these weights are then modified for masked cells (if required)
         and the regridded data are created as the dot product of the
-        weights with the source data. (Note that whilst the `ESMF`
+        weights with the source data. (Note that whilst the `esmpy`
         package is able to also create the regridded data from its
         weights, this feature can't be integrated with the `dask`
         framework that underpins the field's data.)""",
     # regrid Logging
     "{{regrid Logging}}": """**Logging**
 
-        Whether `ESMF` logging is enabled or not is determined by
+        Whether `esmpy` logging is enabled or not is determined by
         `cf.regrid_logging`. If it is logging takes place after every
         call. By default logging is disabled.""",
     # ----------------------------------------------------------------
@@ -392,7 +392,7 @@ _docstring_substitution_definitions = {
                 a point) are skipped, not producing a
                 result. Otherwise an error will be produced if
                 degenerate cells are found, that will be present in
-                the ESMF log files.
+                the esmpy log files.
 
                 For all other regridding methods, degenerate cells are
                 always skipped, regardless of the value of
@@ -431,7 +431,7 @@ _docstring_substitution_definitions = {
                 cell in the regridded field is masked.
 
                 The default value has been chosen empirically as the
-                smallest value that produces the same masks as ESMF
+                smallest value that produces the same masks as esmpy
                 for the use cases defined in the cf test suite.
 
                 Define w_ji as the multiplicative weight that defines
@@ -491,6 +491,22 @@ _docstring_substitution_definitions = {
                 culled. See `dask.optimization.cull` for details.
 
                 .. versionadded:: 3.14.0""",
+    # cfa substitutions
+    "{{cfa substitutions: `dict`}}": """substitutions: `dict`
+                The substitution definitions in a dictionary whose
+                key/value pairs are the file name parts to be
+                substituted and their corresponding substitution text.
+
+                Each substitution definition may be specified with or
+                without the ``${...}`` syntax. For instance, the
+                following are equivalent: ``{'base': 'sub'}``,
+                ``{'${base}': 'sub'}``.""",
+    # cfa base
+    "{{cfa base: `str`}}": """base: `str`
+                The substitution definition to be removed. May be
+                specified with or without the ``${...}`` syntax. For
+                instance, the following are equivalent: ``'base'`` and
+                ``'${base}'``.""",
     # ----------------------------------------------------------------
     # Method description substitutions (4 levels of indentation)
     # ----------------------------------------------------------------
@@ -523,4 +539,19 @@ _docstring_substitution_definitions = {
                             checked. The coordinates check will be
                             carried out, however, if the
                             *check_coordinates* parameter is True.""",
+    # Returns cfa_file_substitutions
+    "{{Returns cfa_file_substitutions}}": """The CFA-netCDF file name substitutions in a dictionary
+                whose key/value pairs are the file name parts to be
+                substituted and their corresponding substitution
+                text.""",
+    # Returns cfa_clear_file_substitutions
+    "{{Returns cfa_clear_file_substitutions}}": """The removed CFA-netCDF file name substitutions in a
+                dictionary whose key/value pairs are the file name
+                parts to be substituted and their corresponding
+                substitution text.""",
+    # Returns cfa_clear_file_substitutions
+    "{{Returns cfa_del_file_substitution}}": """
+                The removed CFA-netCDF file name substitution. If the
+                substitution was not defined then an empty dictionary
+                is returned.""",
 }
