@@ -21,6 +21,7 @@ from os.path import expanduser as _os_path_expanduser
 from os.path import expandvars as _os_path_expandvars
 from os.path import join as _os_path_join
 from os.path import relpath as _os_path_relpath
+from urllib.parse import urlparse
 
 import cfdm
 import netCDF4
@@ -30,7 +31,7 @@ from dask.base import is_dask_collection
 from dask.utils import parse_bytes
 from psutil import virtual_memory
 
-from . import __file__, __version__
+from . import __cfa_version__, __file__, __version__
 from .constants import (
     CONSTANTS,
     OperandBoundsCombination,
@@ -535,9 +536,9 @@ class log_level(ConstantAccess, cfdm.log_level):
 
 
 class regrid_logging(ConstantAccess):
-    """Whether or not to enable `ESMF` regridding logging.
+    """Whether or not to enable `esmpy` regridding logging.
 
-    If it is logging is performed after every call to `ESMF`.
+    If it is logging is performed after every call to `esmpy`.
 
     :Parameters:
 
@@ -1149,6 +1150,33 @@ def CF():
 
 CF.__doc__ = cfdm.CF.__doc__.replace("cfdm.", "cf.")
 
+
+def CFA():
+    """The version of the CFA conventions.
+
+    This indicates which version of the CFA conventions are
+    represented by this release of the cf package, and therefore the
+    version can not be changed.
+
+    .. versionadded:: 3.15.0
+
+    .. seealso:: `cf.CF`
+
+    :Returns:
+
+        `str`
+            The version of the CFA conventions represented by this
+            release of the cf package.
+
+    **Examples**
+
+    >>> cf.CFA()
+    '0.6.2'
+
+    """
+    return __cfa_version__
+
+
 # Module-level alias to avoid name clashes with function keyword
 # arguments (corresponding to 'import atol as cf_atol' etc. in other
 # modules)
@@ -2473,14 +2501,15 @@ def abspath(filename):
     'http://data/archive/file.nc'
 
     """
-    if filename is None:
-        return
+    u = urlparse(filename)
+    scheme = u.scheme
+    if not scheme:
+        return _os_path_abspath(filename)
 
-    u = urllib.parse.urlparse(filename)
-    if u.scheme != "":
-        return filename
+    if scheme == "file":
+        return u.path
 
-    return _os_path_abspath(filename)
+    return filename
 
 
 def relpath(filename, start=None):
@@ -2949,17 +2978,31 @@ def _section(x, axes=None, stop=None, chunks=False, min_step=1):
     return out
 
 
-def _get_module_info(module, try_except=False):
+def _get_module_info(module, alternative_name=False, try_except=False):
     """Helper function for processing modules for cf.environment."""
     if try_except:
+        module_name = None
         try:
             importlib.import_module(module)
+            module_name = module
         except ImportError:
+            if (
+                alternative_name
+            ):  # where a module has a different (e.g. old) name
+                try:
+                    importlib.import_module(alternative_name)
+                    module_name = alternative_name
+                except ImportError:
+                    pass
+
+        if not module_name:
             return ("not available", "")
+    else:
+        module_name = module
 
     return (
-        importlib.import_module(module).__version__,
-        importlib.util.find_spec(module).origin,
+        importlib.import_module(module_name).__version__,
+        importlib.util.find_spec(module_name).origin,
     )
 
 
@@ -2992,7 +3035,7 @@ def environment(display=True, paths=True):
     HDF5 library: 1.10.6
     netcdf library: 4.8.0
     udunits2 library: /home/username/anaconda3/envs/cf-env/lib/libudunits2.so.0
-    ESMF: 8.1.1 /home/username/anaconda3/envs/cf-env/lib/python3.8/site-packages/ESMF/__init__.py
+    esmpy/ESMF: 8.4.1 /home/username/anaconda3/envs/cf-env/lib/python3.8/site-packages/esmpy/__init__.py
     Python: 3.8.10 /home/username/anaconda3/envs/cf-env/bin/python
     dask: 2022.6.0 /home/username/anaconda3/envs/cf-env/lib/python3.8/site-packages/dask/__init__.py
     netCDF4: 1.5.6 /home/username/anaconda3/envs/cf-env/lib/python3.8/site-packages/netCDF4/__init__.py
@@ -3012,7 +3055,7 @@ def environment(display=True, paths=True):
     HDF5 library: 1.10.6
     netcdf library: 4.8.0
     udunits2 library: libudunits2.so.0
-    ESMF: 8.1.1
+    esmpy/ESMF: 8.4.1
     Python: 3.8.10
     dask: 2022.6.0
     netCDF4: 1.5.6
@@ -3035,7 +3078,9 @@ def environment(display=True, paths=True):
         "HDF5 library": (netCDF4.__hdf5libversion__, ""),
         "netcdf library": (netCDF4.__netcdf4libversion__, ""),
         "udunits2 library": (ctypes.util.find_library("udunits2"), ""),
-        "ESMF": _get_module_info("ESMF", try_except=True),
+        "esmpy/ESMF": (
+            _get_module_info("esmpy", alternative_name="ESMF", try_except=True)
+        ),
         # Now Python itself
         "Python": (platform.python_version(), sys.executable),
         # Then Dask (cover first from below as it's important under-the-hood)
@@ -3207,6 +3252,44 @@ def _DEPRECATION_ERROR_FUNCTION_KWARGS(
         )
 
 
+def _DEPRECATION_ERROR_FUNCTION_KWARG_VALUE(
+    func,
+    kwarg,
+    value,
+    message="",
+    version=None,
+    removed_at=None,
+):
+    if removed_at:
+        removed_at = f" and will be removed at version {removed_at}"
+
+    raise DeprecationError(
+        f"Value {value!r} of keyword {kwarg!r} of function {func!r} "
+        f"has been deprecated at version {version} and is no longer "
+        f"available{removed_at}. {message}"
+    )
+
+
+def _DEPRECATION_ERROR_FUNCTION_KWARG(
+    func,
+    kwarg=None,
+    message="",
+    version=None,
+    removed_at=None,
+):
+    if version is None:
+        raise ValueError("Must provide deprecation version, e.g. '3.14.0'")
+
+    if removed_at:
+        removed_at = f" and will be removed at version {removed_at}"
+
+    raise DeprecationError(
+        f"Keyword {kwarg!r} of function {func} has been deprecated "
+        f"at version {version} and is no longer available{removed_at}. "
+        f"{message}"
+    )
+
+
 def _DEPRECATION_ERROR_KWARGS(
     instance,
     method,
