@@ -1,7 +1,6 @@
 from . import Units
 from .data.data import Data
 from .functions import _DEPRECATION_ERROR_FUNCTION_KWARGS
-from .regrid import get_cartesian_coords
 
 
 def relative_vorticity(
@@ -98,8 +97,21 @@ def relative_vorticity(
     v = v.copy()
 
     # Get the X and Y coordinates
-    (u_x_key, u_y_key), (u_x, u_y) = get_cartesian_coords(u, "u", ("X", "Y"))
-    (v_x_key, v_y_key), (v_x, v_y) = get_cartesian_coords(v, "v", ("X", "Y"))
+    u_x_key, u_x = u.dimension_coordinate("X", default=(None, None), item=True)
+    u_y_key, u_y = u.dimension_coordinate("Y", default=(None, None), item=True)
+    v_x_key, v_x = v.dimension_coordinate("X", default=(None, None), item=True)
+    v_y_key, v_y = v.dimension_coordinate("Y", default=(None, None), item=True)
+    if u_x is None:
+        raise ValueError("No unique u-wind X dimension coordinate")
+
+    if u_y is None:
+        raise ValueError("No unique u-wind Y dimension coordinate")
+
+    if v_x is None:
+        raise ValueError("No unique v-wind X dimension coordinate")
+
+    if v_y is None:
+        raise ValueError("No unique v-wind Y dimension coordinate")
 
     if not u_x.equals(v_x) or not u_y.equals(v_y):
         raise ValueError("u and v must be on the same grid.")
@@ -135,11 +147,13 @@ def relative_vorticity(
 
         # Reshape for broadcasting
         u_shape = [1] * u.ndim
-        u_y_index = u.get_data_axes().index(u_y_key)
+        u_y_axis = u.get_data_axes(u_y_key)[0]
+        u_y_index = u.get_data_axes().index(u_y_axis)
         u_shape[u_y_index] = u_y.size
 
         v_shape = [1] * v.ndim
-        v_y_index = v.get_data_axes().index(v_y_key)
+        v_y_axis = u.get_data_axes(v_y_key)[0]
+        v_y_index = v.get_data_axes().index(v_y_axis)
         v_shape[v_y_index] = v_y.size
 
         # Calculate the correction term
@@ -378,13 +392,7 @@ def histogram(*digitized):
     return f.bin("sample_size", digitized=digitized)
 
 
-def curl_xy(
-    fx,
-    fy,
-    x_wrap=None,
-    one_sided_at_boundary=False,
-    radius=None,
-):
+def curl_xy(fx, fy, x_wrap=None, one_sided_at_boundary=False, radius=None):
     r"""Calculate the horizontal curl of an (X, Y) vector.
 
     The horizontal curl is calculated from orthogonal vector component
@@ -553,9 +561,7 @@ def curl_xy(
         r = fx.radius(default=radius)
 
         term1 = (fx * sin_theta).derivative(
-            y_key,
-            wrap=None,
-            one_sided_at_boundary=one_sided_at_boundary,
+            y_key, wrap=None, one_sided_at_boundary=one_sided_at_boundary
         )
 
         term2 = fy.derivative(
@@ -572,15 +578,11 @@ def curl_xy(
         # Cartesian coordinates
         # --------------------------------------------------------
         dfy_dx = fy.derivative(
-            x_key,
-            wrap=x_wrap,
-            one_sided_at_boundary=one_sided_at_boundary,
+            x_key, wrap=x_wrap, one_sided_at_boundary=one_sided_at_boundary
         )
 
         dfx_dy = fx.derivative(
-            y_key,
-            wrap=None,
-            one_sided_at_boundary=one_sided_at_boundary,
+            y_key, wrap=None, one_sided_at_boundary=one_sided_at_boundary
         )
 
         c = dfy_dx - dfx_dy
@@ -594,13 +596,7 @@ def curl_xy(
     return c
 
 
-def div_xy(
-    fx,
-    fy,
-    x_wrap=None,
-    one_sided_at_boundary=False,
-    radius=None,
-):
+def div_xy(fx, fy, x_wrap=None, one_sided_at_boundary=False, radius=None):
     r"""Calculate the horizontal divergence of an (X, Y) vector.
 
     The horizontal divergence is calculated from orthogonal vector
