@@ -1,4 +1,5 @@
 import logging
+from copy import deepcopy
 from operator import __and__ as operator_and
 from operator import __or__ as operator_or
 
@@ -19,6 +20,9 @@ from .units import Units
 
 logger = logging.getLogger(__name__)
 
+# Alias for builtin set, since there is a 'set' function
+builtin_set = set
+
 
 class Query:
     '''Encapsulate a condition for subsequent evaluation.
@@ -62,7 +66,7 @@ class Query:
     ``'set'``  A "member of set" condition
     =========  ===================================
 
-    **Complex conditions**
+    **Compound queries**
 
     Multiple conditions may be combined with the Python bitwise "and"
     (`&`) and "or" (`|`) operators to form a new `Query` object.
@@ -182,9 +186,9 @@ class Query:
 
     isquery = True
 
-    @_deprecated_kwarg_check("exact")
+    @_deprecated_kwarg_check("exact", version="3.0.0", removed_at="4.0.0")
     def __init__(self, operator, value, units=None, attr=None, exact=True):
-        """**Initialization**
+        """**Initialisation**
 
         :Parameters:
 
@@ -235,6 +239,8 @@ class Query:
 
         self._bitwise_operator = None
 
+        self.query_type = operator
+
         self._NotImplemented_RHS_Data_op = True
 
     def __deepcopy__(self, memo):
@@ -265,6 +271,10 @@ class Query:
     def __and__(self, other):
         """The binary bitwise operation ``&``
 
+        Combine two queries with a logical And operation. If the
+        `!value` of both queries is the same then it will be retained on
+        the compound query.
+
         x.__and__(y) <==> x&y
 
         """
@@ -272,10 +282,19 @@ class Query:
         new = Q.__new__(Q)
 
         new._operator = None
-        new._compound = (self, other)
+        new._compound = (self.copy(), other.copy())
         new._bitwise_operator = operator_and
         new._attr = ()
 
+        # If the value of the two queries is the same then retain it
+        # on the compound query
+        value0 = self._value
+        value1 = other._value
+        if value0 is None or value1 is None or value0 != value1:
+            new._value = None
+        else:
+            new._value = deepcopy(value0)
+
         new._NotImplemented_RHS_Data_op = True
 
         return new
@@ -291,6 +310,10 @@ class Query:
     def __or__(self, other):
         """The binary bitwise operation ``|``
 
+        Combine two queries with a logical Or operation. If the
+        `!value` of both queries is the same then it will be retained on
+        the compound query.
+
         x.__or__(y) <==> x|y
 
         """
@@ -302,6 +325,15 @@ class Query:
         new._bitwise_operator = operator_or
         new._attr = ()
 
+        # If the value of the two queries is the same then retain it
+        # on the compound query
+        value0 = self._value
+        value1 = other._value
+        if value0 is None or value1 is None or value0 != value1:
+            new._value = None
+        else:
+            new._value = deepcopy(value0)
+
         new._NotImplemented_RHS_Data_op = True
 
         return new
@@ -396,10 +428,13 @@ class Query:
         AttributeError: Compound query doesn't have attribute 'value'
 
         """
-        if not self._compound:
-            return self._value
+        value = self._value
+        if value is None:
+            raise AttributeError(
+                "Compound query doesn't have attribute 'value'"
+            )
 
-        raise AttributeError("Compound query doesn't have attribute 'value'")
+        return value
 
     def addattr(self, attr):
         """Return a `Query` object with a new left hand side operand
@@ -462,7 +497,18 @@ class Query:
         >>> r = q.copy()
 
         """
-        return self  # TODO
+        Q = type(self)
+        new = Q.__new__(Q)
+
+        d = self.__dict__.copy()
+        new.__dict__ = d
+
+        if d["_compound"]:
+            d["_compound"] = deepcopy(d["_compound"])
+        else:
+            d["_value"] = deepcopy(d["_value"])
+
+        return new
 
     @_display_or_return
     def dump(self, display=True):
@@ -486,7 +532,7 @@ class Query:
         """
         return str(self)
 
-    @_deprecated_kwarg_check("traceback")
+    @_deprecated_kwarg_check("traceback", version="3.0.0", removed_at="4.0.0")
     @_manage_log_level_via_verbosity
     def equals(self, other, verbose=None, traceback=False):
         """True if two `Query` objects are the same."""
@@ -678,13 +724,6 @@ class Query:
 
             return (x < value[0]) | (x > value[1])
 
-        #        if operator == 'contains':
-        #            _contain = getattr(x, '__query_contains__', None)
-        #            if _contain is not None:
-        #                return _contain(value)
-        #            else:
-        #                return x == value
-
         if operator == "set":
             if isinstance(x, str):
                 for v in value:
@@ -721,6 +760,147 @@ class Query:
         """
         print(_inspect(self))  # pragma: no cover
 
+    def iscontains(self):
+        """Return True if the query is a "cell contains" condition.
+
+        .. versionadded:: 3.14.0
+
+        .. seealso:: `cf.contains`
+
+        :Returns:
+
+            `bool`
+                Whether or not the query is a "cell contains"
+                condition.
+
+        **Examples**
+
+        >>> q = cf.contains(15)
+        >>> q
+        <CF Query: [lower_bounds(le 15) & upper_bounds(ge 15)]>
+        >>> q.iscontains()
+        True
+        >>> q |= cf.lt(6)
+        >>> q.iscontains()
+        False
+
+        >>> r = cf.wi(6, 10)
+        >>> r.iscontains()
+        False
+
+        """
+        if not (
+            self._compound
+            and self._value is not None
+            and self._bitwise_operator is operator_and
+        ):
+            return False
+
+        sig = builtin_set()
+        for q in self._compound:
+            if q._compound:
+                # A compound query of compound queries is not a "cell
+                # contains" condition
+                break
+
+            sig.add((q.attr, q.operator))
+
+        return sig == builtin_set(
+            ((("lower_bounds",), "le"), (("upper_bounds",), "ge"))
+        )
+
+    def set_condition_units(self, units):
+        """Set units of condition values in-place.
+
+        .. versionadded:: TODO
+
+        :Parameters:
+
+            units: `str` or `Units`
+
+                The units to be set on all condition values.
+
+        :Returns:
+
+            `None`
+
+        **Examples**
+
+        >>> q = cf.lt(9)
+        >>> q
+        <CF Query: (lt 9)>
+        >>> q.set_condition_units('km')
+        >>> q
+        <CF Query: (lt 9 km)>
+        >>> q.set_condition_units('seconds')
+            ...
+        ValueError: Units <Units: seconds> are not equivalent to query condition units <Units: m>
+
+        >>> q = cf.lt(9, units='m')
+        >>> q
+        <CF Query: (lt 9 m)>
+        >>> q.set_condition_units('km')
+        >>> q
+        <CF Query: (lt 0.009 km)>
+
+        >>> q = cf.lt(9)
+        >>> r = cf.ge(3000, units='m')
+        >>> s = q & r
+        >>> s
+        <CF Query: [(lt 9) & (ge 3000 m)]>
+        >>> s.set_condition_units('km')
+        >>> s
+        <CF Query: [(lt 9 km) & (ge 3 km)]>
+        >>> q
+        <CF Query: (lt 9)>
+        >>> r
+        <CF Query: (ge 3000 m)>
+
+        """
+
+        def get_and_set_value_units(v, u):
+            """Helper method to simplify setting of specified units."""
+            v_units = getattr(v, "Units", None)
+            if v_units is None:  # Value 'v' has no units
+                v = Data(v, units=u)
+            else:  # Value 'v' already has units
+                try:
+                    v = v.copy()
+                    v.Units = u
+                except ValueError:
+                    raise ValueError(
+                        f"Units {u!r} are not equivalent to "
+                        f"query condition units {v_units!r}"
+                    )
+
+            return v
+
+        units = Units(units)
+
+        compound = self._compound
+        if compound:
+            for r in compound:
+                r.set_condition_units(units)
+            return
+
+        value = self._value
+        if value is None:
+            return
+
+        if self.operator in ("wi", "wo", "set"):
+            # Value is a sequence of things that may or may not
+            # already have units
+            new_values = []
+            for v in value:
+                v = get_and_set_value_units(v, units)
+                new_values.append(v)
+
+            value = new_values
+        else:
+            value = get_and_set_value_units(value, units)
+
+        self._value = value
+
     # ----------------------------------------------------------------
     # Deprecated attributes and methods
     # ----------------------------------------------------------------
@@ -732,12 +912,18 @@ class Query:
 
         """
         _DEPRECATION_ERROR_ATTRIBUTE(
-            self, "exact", "Use 're.compile' objects instead."
+            self,
+            "exact",
+            "Use 're.compile' objects instead.",
+            version="3.0.0",
+            removed_at="4.0.0",
         )  # pragma: no cover
 
     def equivalent(self, other, traceback=False):
         """Deprecated at version 3.0.0."""
-        _DEPRECATION_ERROR_FUNCTION(self, "equivalent")
+        _DEPRECATION_ERROR_FUNCTION(
+            self, "equivalent", version="3.0.0", removed_at="4.0.0"
+        )
 
 
 # --------------------------------------------------------------------
@@ -1207,8 +1393,9 @@ def contains(value, units=None):
 
     .. versionadded:: 3.0.0
 
-    .. seealso:: `cf.cellsize`, `cf.cellge`, `cf.cellgt`, `cf.cellne`,
-                 `cf.cellle`, `cf.celllt`, `cf.cellwi`, `cf.cellwo`
+    .. seealso:: `cf.Query.iscontains`, `cf.cellsize`, `cf.cellge`,
+                 `cf.cellgt`, `cf.cellne`, `cf.cellle`, `cf.celllt`,
+                 `cf.cellwi`, `cf.cellwo`
 
     :Parameters:
 
@@ -1229,10 +1416,18 @@ def contains(value, units=None):
 
     **Examples**
 
-    >>> cf.contains(8)
+    >>> q = cf.contains(8)
+    >>> q
     <CF Query: [lower_bounds(le 8) & upper_bounds(ge 8)]>
-    >>> cf.contains(30, 'degrees_east')
+    >>> q.value
+    8
+
+    >>> q = cf.contains(30, 'degrees_east')
+    >>> q
     <CF Query: [lower_bounds(le 30 degrees_east) & upper_bounds(ge 30 degrees_east)]>
+    >>> q.value
+    <CF Data(): 30 degrees_east>
+
     >>> cf.contains(cf.Data(10, 'km'))
     <CF Query: [lower_bounds(le 10 km) & upper_bounds(ge 10 km)]>
 
@@ -2000,7 +2195,10 @@ def dtge(*args, **kwargs):
 
     """
     _DEPRECATION_ERROR_FUNCTION(
-        "dtge", "Use 'cf.ge' with a datetime object value instead."
+        "dtge",
+        "Use 'cf.ge' with a datetime object value instead.",
+        version="3.0.0",
+        removed_at="4.0.0",
     )  # pragma: no cover
 
 
@@ -2011,7 +2209,10 @@ def dtgt(*args, **kwargs):
 
     """
     _DEPRECATION_ERROR_FUNCTION(
-        "dtgt", "Use 'cf.gt' with a datetime object value instead."
+        "dtgt",
+        "Use 'cf.gt' with a datetime object value instead.",
+        version="3.0.0",
+        removed_at="4.0.0",
     )  # pragma: no cover
 
 
@@ -2022,7 +2223,10 @@ def dtle(*args, **kwargs):
 
     """
     _DEPRECATION_ERROR_FUNCTION(
-        "dtle", "Use 'cf.le' with a datetime object value instead."
+        "dtle",
+        "Use 'cf.le' with a datetime object value instead.",
+        version="3.0.0",
+        removed_at="4.0.0",
     )  # pragma: no cover
 
 
@@ -2033,7 +2237,10 @@ def dtlt(*args, **kwargs):
 
     """
     _DEPRECATION_ERROR_FUNCTION(
-        "dtlt", "Use 'cf.lt' with a datetime object value instead."
+        "dtlt",
+        "Use 'cf.lt' with a datetime object value instead.",
+        version="3.0.0",
+        removed_at="4.0.0",
     )  # pragma: no cover
 
 
@@ -2044,7 +2251,10 @@ def dteq(*args, **kwargs):
 
     """
     _DEPRECATION_ERROR_FUNCTION(
-        "dteq", "Use 'cf.eq' with a datetime object value instead."
+        "dteq",
+        "Use 'cf.eq' with a datetime object value instead.",
+        version="3.0.0",
+        removed_at="4.0.0",
     )  # pragma: no cover
 
 
@@ -2055,7 +2265,10 @@ def dtne(*args, **kwargs):
 
     """
     _DEPRECATION_ERROR_FUNCTION(
-        "dtne", "Use 'cf.ne' with a datetime object value instead."
+        "dtne",
+        "Use 'cf.ne' with a datetime object value instead.",
+        version="3.0.0",
+        removed_at="4.0.0",
     )  # pragma: no cover
 
 
@@ -2066,5 +2279,8 @@ def contain(value, units=None, attr=None):
 
     """
     _DEPRECATION_ERROR_FUNCTION(
-        "cf.contain", "Use function 'cf.contains' instead"
+        "cf.contain",
+        "Use function 'cf.contains' instead",
+        version="3.0.0",
+        removed_at="4.0.0",
     )  # pragma: no cover
