@@ -1,24 +1,7 @@
 import cfdm
+import netCDF4
 import numpy as np
-
-"""
-TODOCFA: remove aggregation_* properties from constructs
-
-TODOCFA: Create auxiliary coordinates from non-standardised terms
-
-TODOCFA: Reference instruction variables (and/or set as
-         "do_not_create_field")
-
-TODOCFA: Create auxiliary coordinates from non-standardised terms
-
-TODOCFA: Consider scanning for cfa variables to the top (e.g. where
-         scanning for geometry varables is). This will probably need a
-         change in cfdm so that a customizable hook can be overlaoded
-         (like `_customize_read_vars` does).
-
-TODOCFA: What about groups/netcdf_flattener?
-
-"""
+from packaging.version import Version
 
 
 class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
@@ -144,7 +127,9 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
                 parent_ncvar=parent_ncvar,
             )
 
-        # Still here? Then we have a CFA variable.
+        # ------------------------------------------------------------
+        # Still here? Then we have a CFA-netCDF variable.
+        # ------------------------------------------------------------
         g = self.read_vars
 
         ncdimensions = g["variable_attributes"][ncvar][
@@ -168,6 +153,7 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
         uncompress_override=None,
         parent_ncvar=None,
         coord_ncvar=None,
+        cfa_term=None,
     ):
         """Create data for a netCDF or CFA-netCDF variable.
 
@@ -176,7 +162,8 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
         :Parameters:
 
             ncvar: `str`
-                The name of the netCDF variable that contains the data.
+                The name of the netCDF variable that contains the
+                data. See the *cfa_term* parameter.
 
             construct: optional
 
@@ -184,20 +171,24 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
 
             uncompress_override: `bool`, optional
 
-            parent_ncvar: `str`, optional
-
             coord_ncvar: `str`, optional
 
-                .. versionadded:: TODO
+            cfa_term: `dict`, optional
+                The name of a non-standard aggregation instruction
+                term from which to create the data. If set then
+                *ncvar* must be the value of the term in the
+                ``aggregation_data`` attribute.
+
+                .. versionadded:: 3.15.0
 
         :Returns:
 
             `Data`
 
         """
-        if not self._is_cfa_variable(ncvar):
+        if not cfa_term and not self._is_cfa_variable(ncvar):
             # Create data for a normal netCDF variable
-            return super()._create_data(
+            data = super()._create_data(
                 ncvar=ncvar,
                 construct=construct,
                 unpacked_dtype=unpacked_dtype,
@@ -206,25 +197,80 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
                 coord_ncvar=coord_ncvar,
             )
 
-        # ------------------------------------------------------------
-        # Still here? Then create data for a CFA-netCDF variable
-        # ------------------------------------------------------------
-        cfa_array, kwargs = self._create_cfanetcdfarray(
-            ncvar,
-            unpacked_dtype=unpacked_dtype,
-            coord_ncvar=coord_ncvar,
-        )
+            # Set the CFA write status to True when there is exactly
+            # one dask chunk
+            if data.npartitions == 1:
+                data._cfa_set_write(True)
 
-        # Return the data
-        return self._create_Data(
+            self._cache_data_elements(data, ncvar)
+
+            return data
+
+        # ------------------------------------------------------------
+        # Still here? Create data for a CFA variable
+        # ------------------------------------------------------------
+        if construct is not None:
+            # Remove the aggregation attributes from the construct
+            self.implementation.del_property(
+                construct, "aggregated_dimensions", None
+            )
+            aggregated_data = self.implementation.del_property(
+                construct, "aggregated_data", None
+            )
+        else:
+            aggregated_data = None
+
+        if cfa_term:
+            term, term_ncvar = tuple(cfa_term.items())[0]
+            cfa_array, kwargs = self._create_cfanetcdfarray_term(
+                ncvar, term, term_ncvar
+            )
+        else:
+            cfa_array, kwargs = self._create_cfanetcdfarray(
+                ncvar,
+                unpacked_dtype=unpacked_dtype,
+                coord_ncvar=coord_ncvar,
+            )
+
+        data = self._create_Data(
             cfa_array,
             ncvar,
             units=kwargs["units"],
             calendar=kwargs["calendar"],
         )
 
+        # Note: We don't cache elements from CFA variables, because
+        #       the data are in fragment files which have not been
+        #       opened; and may not not even be openable, such as
+        #       could be the case if a fragement was on tape storage.
+
+        # Set the CFA write status to True iff each non-aggregated
+        # axis has exactly one dask storage chunk
+        if cfa_term:
+            data._cfa_set_term(True)
+        else:
+            cfa_write = True
+            for n, numblocks in zip(
+                cfa_array.get_fragment_shape(), data.numblocks
+            ):
+                if n == 1 and numblocks > 1:
+                    # Note: 'n == 1' is True for non-aggregated axes
+                    cfa_write = False
+                    break
+
+            data._cfa_set_write(cfa_write)
+
+            # Store the 'aggregated_data' attribute
+            if aggregated_data:
+                data.cfa_set_aggregated_data(aggregated_data)
+
+            # Store the file substitutions
+            data.cfa_update_file_substitutions(kwargs.get("substitutions"))
+
+        return data
+
     def _is_cfa_variable(self, ncvar):
-        """Return True if *ncvar* is a CFA variable.
+        """Return True if *ncvar* is a CFA aggregated variable.
 
         .. versionadded:: 3.14.0
 
@@ -240,39 +286,11 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
 
         """
         g = self.read_vars
-
-        if not g["cfa"] or ncvar in g["external_variables"]:
-            return False
-
-        attributes = g["variable_attributes"][ncvar]
-
-        # TODOCFA: test on the version of CFA given by g["cfa"]. See
-        #          also `_customize_read_vars`.
-        cfa = "aggregated_dimensions" in attributes
-        if cfa:
-            # TODOCFA: Modify this message for v4.0.0
-            raise ValueError(
-                "The reading of CFA files has been temporarily disabled, "
-                "but will return for CFA-0.6 files at version 4.0.0. "
-                "CFA-0.4 functionality is still available at version 3.13.1."
-            )
-
-            # TODOCFA: The 'return' remains when the exception is
-            #          removed at v4.0.0.
-            return True
-
-        cfa_04 = attributes.get("cf_role") == "cfa_variable"
-        if cfa_04:
-            # TODOCFA: Modify this message for v4.0.0.
-            raise ValueError(
-                "The reading of CFA-0.4 files was permanently disabled at "
-                "version 3.14.0. However, CFA-0.4 functionality is "
-                "still available at version 3.13.1. "
-                "The reading and writing of CFA-0.6 files will become "
-                "available at version 4.0.0."
-            )
-
-        return False
+        return (
+            g["cfa"]
+            and ncvar in g["cfa_aggregated_data"]
+            and ncvar not in g["external_variables"]
+        )
 
     def _create_Data(
         self,
@@ -295,11 +313,6 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
             ncvar: `str`
                 The netCDF variable containing the array.
 
-            ncdimensions: sequence of `str`, optional
-                The netCDF dimensions spanned by the array.
-
-                .. versionadded:: 3.14.0
-
             units: `str`, optional
                 The units of *array*. By default, or if `None`, it is
                 assumed that there are no units.
@@ -308,6 +321,11 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
                 The calendar of *array*. By default, or if `None`, it is
                 assumed that there is no calendar.
 
+            ncdimensions: sequence of `str`, optional
+                The netCDF dimensions spanned by the array.
+
+                .. versionadded:: 3.14.0
+
             kwargs: optional
                 Extra parameters to pass to the initialisation of the
                 returned `Data` object.
@@ -335,51 +353,68 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
             chunks=chunks,
             **kwargs,
         )
-        self._cache_data_elements(data, ncvar)
 
         return data
 
-    def _customize_read_vars(self):
-        """Customize the read parameters.
+    def _customise_read_vars(self):
+        """Customise the read parameters.
+
+        Take the opportunity to apply CFA updates to
+        `read_vars['variable_dimensions']` and
+        `read_vars['do_not_create_field']`.
 
         .. versionadded:: 3.0.0
 
         """
-        super()._customize_read_vars()
-
+        super()._customise_read_vars()
         g = self.read_vars
 
+        if not g["cfa"]:
+            return
+
+        g["cfa_aggregated_data"] = {}
+        g["cfa_aggregation_instructions"] = {}
+        g["cfa_file_substitutions"] = {}
+
         # ------------------------------------------------------------
-        # Find out if this is a CFA file
+        # Still here? Then this is a CFA-netCDF file
         # ------------------------------------------------------------
-        g["cfa"] = "CFA" in g["global_attributes"].get("Conventions", ())
+        if g["CFA_version"] < Version("0.6.2"):
+            raise ValueError(
+                f"Can't read file {g['filename']} that uses obsolete "
+                f"CFA conventions version CFA-{g['CFA_version']}. "
+                "(Note that cf version 3.13.1 can be used to read and "
+                "write CFA-0.4 files.)"
+            )
 
-        if g["cfa"]:
-            attributes = g["variable_attributes"]
-            dimensions = g["variable_dimensions"]
+        # Get the directory of the CFA-netCDF file being read
+        from os.path import abspath
+        from pathlib import PurePath
 
-            # Do not create fields from CFA private
-            # variables. TODOCFA: get private variables from
-            # CFANetCDFArray instances
-            for ncvar in g["variables"]:
-                if attributes[ncvar].get("cf_role", None) == "cfa_private":
-                    g["do_not_create_field"].add(ncvar)
+        g["cfa_dir"] = PurePath(abspath(g["filename"])).parent
 
-            for ncvar, ncdims in tuple(dimensions.items()):
-                if ncdims != ():
-                    continue
+        # Process the aggregation instruction variables, and the
+        # aggregated dimensions.
+        dimensions = g["variable_dimensions"]
+        attributes = g["variable_attributes"]
 
-                if not (
-                    ncvar not in g["external_variables"]
-                    and "aggregated_dimensions" in attributes[ncvar]
-                ):
-                    continue
+        for ncvar, attributes in attributes.items():
+            if "aggregated_dimensions" not in attributes:
+                # This is not an aggregated variable
+                continue
 
-                ncdimensions = attributes[ncvar][
-                    "aggregated_dimensions"
-                ].split()
-                if ncdimensions:
-                    dimensions[ncvar] = tuple(map(str, ncdimensions))
+            # Set the aggregated variable's dimensions as its
+            # aggregated dimensions
+            ncdimensions = attributes["aggregated_dimensions"].split()
+            dimensions[ncvar] = tuple(map(str, ncdimensions))
+
+            # Do not create fields/domains from aggregation
+            # instruction variables
+            parsed_aggregated_data = self._cfa_parse_aggregated_data(
+                ncvar, attributes.get("aggregated_data")
+            )
+            for term_ncvar in parsed_aggregated_data.values():
+                g["do_not_create_field"].add(term_ncvar)
 
     def _cache_data_elements(self, data, ncvar):
         """Cache selected element values.
@@ -464,19 +499,30 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
         ncvar,
         unpacked_dtype=False,
         coord_ncvar=None,
+        term=None,
     ):
         """Create a CFA-netCDF variable array.
 
-        .. versionadded:: (cfdm) 1.10.0.1
+        .. versionadded:: 3.14.0
 
         :Parameters:
 
             ncvar: `str`
+                The name of the CFA-netCDF aggregated variable. See
+                the *term* parameter.
 
             unpacked_dtype: `False` or `numpy.dtype`, optional
 
             coord_ncvar: `str`, optional
 
+            term: `str`, optional
+                The name of a non-standard aggregation instruction
+                term from which to create the array. If set then
+                *ncvar* must be the value of the non-standard term in
+                the ``aggregation_data`` attribute.
+
+                .. versionadded:: 3.15.0
+
         :Returns:
 
             (`CFANetCDFArray`, `dict`)
@@ -484,6 +530,8 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
                 kwargs used to create it.
 
         """
+        g = self.read_vars
+
         # Get the kwargs needed to instantiate a general NetCDFArray
         # instance
         kwargs = self._create_netcdfarray(
@@ -493,17 +541,97 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
             return_kwargs_only=True,
         )
 
-        # Get rid of the incorrect shape
+        # Get rid of the incorrect shape of (). This will end up
+        # getting set correctly by the CFANetCDFArray instance.
         kwargs.pop("shape", None)
 
-        # Add the aggregated_data attribute (that can be used by
-        # dask.base.tokenize).
-        kwargs["instructions"] = self.read_vars["variable_attributes"][
-            ncvar
-        ].get("aggregated_data")
+        aggregated_data = g["cfa_aggregated_data"][ncvar]
 
-        # Use the kwargs to create a specialised CFANetCDFArray
+        standardised_terms = ("location", "file", "address", "format")
+
+        instructions = []
+        aggregation_instructions = {}
+        for t, term_ncvar in aggregated_data.items():
+            if t not in standardised_terms:
+                continue
+
+            aggregation_instructions[t] = g["cfa_aggregation_instructions"][
+                term_ncvar
+            ]
+            instructions.append(f"{t}: {term_ncvar}")
+
+            if t == "file":
+                kwargs["substitutions"] = g["cfa_file_substitutions"].get(
+                    term_ncvar
+                )
+
+        kwargs["x"] = aggregation_instructions
+        kwargs["instructions"] = " ".join(sorted(instructions))
+
+        # Use the kwargs to create a CFANetCDFArray instance
+        array = self.implementation.initialise_CFANetCDFArray(**kwargs)
+
+        return array, kwargs
+
+    def _create_cfanetcdfarray_term(
+        self,
+        parent_ncvar,
+        term,
+        ncvar,
+    ):
+        """Create a CFA-netCDF variable array.
+
+        .. versionadded:: 3.14.0
+
+        :Parameters:
+
+            parent_ncvar: `str`
+                The name of the CFA-netCDF aggregated variable. See
+                the *term* parameter.
+
+            term: `str`, optional
+                The name of a non-standard aggregation instruction
+                term from which to create the array. If set then
+                *ncvar* must be the value of the non-standard term in
+                the ``aggregation_data`` attribute.
+
+                .. versionadded:: 3.15.0
+
+            ncvar: `str`
+                The name of the CFA-netCDF aggregated variable. See
+                the *term* parameter.
+
+        :Returns:
+
+            (`CFANetCDFArray`, `dict`)
+                The new `NetCDFArray` instance and dictionary of the
+                kwargs used to create it.
+
+        """
+        g = self.read_vars
+
+        # Get the kwargs needed to instantiate a general NetCDFArray
         # instance
+        kwargs = self._create_netcdfarray(
+            ncvar,
+            return_kwargs_only=True,
+        )
+
+        instructions = []
+        aggregation_instructions = {}
+        for t, term_ncvar in g["cfa_aggregated_data"][parent_ncvar].items():
+            if t in ("location", term):
+                aggregation_instructions[t] = g[
+                    "cfa_aggregation_instructions"
+                ][term_ncvar]
+                instructions.append(f"{t}: {ncvar}")
+
+        kwargs["term"] = term
+        kwargs["dtype"] = aggregation_instructions[term].dtype
+        kwargs["x"] = aggregation_instructions
+        kwargs["instructions"] = " ".join(sorted(instructions))
+
+        # Use the kwargs to create a CFANetCDFArray instance
         array = self.implementation.initialise_CFANetCDFArray(**kwargs)
 
         return array, kwargs
@@ -572,3 +700,201 @@ class NetCDFRead(cfdm.read_write.netcdf.NetCDFRead):
             chunks = chunks2
 
         return chunks
+
+    def _customise_field_ancillaries(self, parent_ncvar, f):
+        """Create customised field ancillary constructs.
+
+        This method currently creates:
+
+        * Field ancillary constructs derived from non-standardised
+          terms in CFA aggregation instructions. Each construct spans
+          the same domain axes as the parent field construct.
+          Constructs are never created for `Domain` instances.
+
+        .. versionadded:: 3.15.0
+
+        :Parameters:
+
+            parent_ncvar: `str`
+                The netCDF variable name of the parent variable.
+
+            f: `Field`
+                The parent field construct.
+
+        :Returns:
+
+            `dict`
+                A mapping of netCDF variable names to newly-created
+                construct identifiers.
+
+        **Examples**
+
+        >>> n._customise_field_ancillaries('tas', f)
+        {}
+
+        >>> n._customise_field_ancillaries('pr', f)
+        {'tracking_id': 'fieldancillary1'}
+
+        """
+        if not self._is_cfa_variable(parent_ncvar):
+            return {}
+
+        # ------------------------------------------------------------
+        # Still here? Then we have a CFA-netCDF variable: Loop round
+        # the aggregation instruction terms and convert each
+        # non-standard term into a field ancillary construct that
+        # spans the same domain axes as the parent field.
+        # ------------------------------------------------------------
+        g = self.read_vars
+
+        standardised_terms = ("location", "file", "address", "format")
+
+        out = {}
+        for term, term_ncvar in g["cfa_aggregated_data"][parent_ncvar].items():
+            if term in standardised_terms:
+                continue
+
+            if g["variables"][term_ncvar].ndim != f.ndim:
+                # Can only create field ancillaries with the same rank
+                # as the field
+                continue
+
+            # Still here? Then we've got a non-standard aggregation
+            #             term from which we can create a field
+            #             ancillary construct.
+            anc = self.implementation.initialise_FieldAncillary()
+
+            self.implementation.set_properties(
+                anc, g["variable_attributes"][term_ncvar]
+            )
+            anc.set_property("long_name", term)
+
+            # Store the term name as the 'id' attribute. This will be
+            # used as the term name if the field field ancillary is
+            # written to disk as a non-standard CFA term.
+            anc.id = term
+
+            data = self._create_data(
+                parent_ncvar, anc, cfa_term={term: term_ncvar}
+            )
+
+            self.implementation.set_data(anc, data, copy=False)
+            self.implementation.nc_set_variable(anc, term_ncvar)
+
+            key = self.implementation.set_field_ancillary(
+                f,
+                anc,
+                axes=self.implementation.get_field_data_axes(f),
+                copy=False,
+            )
+            out[term_ncvar] = key
+
+        return out
+
+    def _cfa_parse_aggregated_data(self, ncvar, aggregated_data):
+        """Parse a CFA-netCDF ``aggregated_data`` attribute.
+
+        .. versionadded:: 3.15.0
+
+        :Parameters:
+
+            ncvar: `str`
+                The netCDF variable name.
+
+            aggregated_data: `str` or `None`
+                The CFA-netCDF ``aggregated_data`` attribute.
+
+        :Returns:
+
+            `dict`
+                The parsed attribute.
+
+        """
+        if not aggregated_data:
+            return {}
+
+        g = self.read_vars
+        aggregation_instructions = g["cfa_aggregation_instructions"]
+        variable_attributes = g["variable_attributes"]
+
+        out = {}
+        for x in self._parse_x(
+            ncvar,
+            aggregated_data,
+            keys_are_variables=True,
+        ):
+            term, term_ncvar = tuple(x.items())[0]
+            term_ncvar = term_ncvar[0]
+            out[term] = term_ncvar
+
+            if term_ncvar in aggregation_instructions:
+                # Already processed this term
+                continue
+
+            array = g["variables"][term_ncvar][...]
+            aggregation_instructions[term_ncvar] = self._cfa_conform_array(
+                array
+            )
+
+            if term == "file":
+                # Find URI substitutions that may be stored in the
+                # CFA file instruction variable's "substitutions"
+                # attribute
+                subs = variable_attributes[term_ncvar].get(
+                    "substitutions",
+                )
+                if subs:
+                    # Convert the string "${base}: value" to the
+                    # dictionary {"${base}": "value"}
+                    s = subs.split()
+                    subs = {
+                        base[:-1]: sub for base, sub in zip(s[::2], s[1::2])
+                    }
+
+                    # Apply user-defined substitutions, which take
+                    # precedence over those defined in the file.
+                    subs.update(g["cfa_options"].get("substitutions", {}))
+                    g["cfa_file_substitutions"][term_ncvar] = subs
+
+        g["cfa_aggregated_data"][ncvar] = out
+        return out
+
+    def _cfa_conform_array(self, array):
+        """Conform an array so that it is suitable for CFA processing.
+
+        .. versionadded: 3.15.0
+
+        :Parameters:
+
+            array: `np.ndarray`
+                The array to conform.
+
+        :Returns:
+
+            array: `np.ndarray`
+                The conformed array.
+
+        """
+        if isinstance(array, str):
+            # string
+            return np.array(array, dtype=f"S{len(array)}").astype("U")
+
+        kind = array.dtype.kind
+        if kind == "O":
+            # string
+            return array.astype("U")
+
+        if kind in "SU":
+            # char
+            if kind == "U":
+                array = array.astype("S")
+
+            array = netCDF4.chartostring(array)
+            shape = array.shape
+            array = np.array([x.rstrip() for x in array.flat], dtype="S")
+            array = np.reshape(array, shape)
+            array = np.ma.masked_where(array == b"", array)
+            return array.astype("U")
+
+        # number
+        return array
